from microservice_template_core.tools.logger import get_logger
from microservice_template_core.settings import KafkaConfig
import traceback
from prometheus_client import Summary
from confluent_kafka import Producer
import ujson as json
import datetime

logger = get_logger()


class KafkaProduceMessages(object):
    KAFKA_PRODUCER_START = Summary('kafka_confluent_producer_start_latency_seconds', 'Time spent starting Kafka producer')
    KAFKA_PRODUCE_MESSAGES = Summary('kafka_confluent_produce_messages_latency_seconds', 'Time spent processing produce to Kafka')

    def __init__(self):
        self.producer = self.init_producer()

    @staticmethod
    @KAFKA_PRODUCER_START.time()
    def init_producer():
        try:
            producer = Producer({'bootstrap.servers': KafkaConfig.KAFKA_SERVERS})

            return producer
        except Exception as err:
            logger.error(
                msg=f"Can`t connect to Kafka cluster - {KafkaConfig.KAFKA_SERVERS}\nError: {err}\nTrace: {traceback.format_exc()}"
            )
            return None

    def default_converter(self):
        if isinstance(self, (datetime.date, datetime.datetime)):
            return self.isoformat()

    @staticmethod
    def delivery_report(err, msg):
        """ Called once for each message produced to indicate delivery result.
            Triggered by poll() or flush(). """
        if err is not None:
            logger.error('Message delivery failed: {}'.format(err))
        else:
            logger.debug('Message delivered to {} [{}]'.format(msg.topic(), msg.partition()))

    @KAFKA_PRODUCE_MESSAGES.time()
    def produce_message(self, topic, message):
        logger.debug(
            msg=f"Start producing message to topic - {topic}\nBody: {message}"
        )
        try:
            self.producer.produce(
                topic,
                json.dumps(message, default=self.default_converter).encode(),
                callback=self.delivery_report
            )

            self.producer.flush()
        except Exception as err:
            logger.error(
                msg=f"Can`t push message to - {topic}\nBody: {message}\nError: {err}\nTrace: {traceback.format_exc()}"
            )



from microservice_template_core.tools.logger import get_logger
from microservice_template_core.settings import ServiceConfig, KafkaConfig
from kafka import KafkaConsumer

logger = get_logger()


class KafkaConsumeMessages(object):
    def __init__(self, kafka_topic):
        self.kafka_topic = kafka_topic

    def start_consumer(self):
        """
        Start consumer
        """

        logger.debug(f"Initializing Kafka Consumer - {ServiceConfig.SERVICE_NAME}")
        consumer = KafkaConsumer(
            self.kafka_topic,
            group_id=ServiceConfig.SERVICE_NAME,
            bootstrap_servers=KafkaConfig.KAFKA_SERVERS
        )

        return consumer



from flask_restx import Api
from jwt import exceptions as jwt_exception
from flask_jwt_extended import exceptions as jwt_extended_exception
from flask_jwt_extended import get_jwt_identity
from werkzeug.exceptions import NotFound
api = Api()


@api.errorhandler(NotFound)
def handle_no_result_exception(e):
    ''' Return a custom not found error message and 404 status code '''
    return {'message': 'No result found'}, 404


@api.errorhandler
def default_error_handler(e):
    message = 'An unhandled exception occurred.'
    return {'message': message}, 500


@api.errorhandler(jwt_extended_exception.NoAuthorizationError)
def handle_auth_error(e):
    return {'message': str(e)}, 401


@api.errorhandler(jwt_extended_exception.CSRFError)
def handle_auth_error(e):
    return {'message': str(e)}, 401


@api.errorhandler(jwt_exception.ExpiredSignatureError)
def handle_expired_error(e):
    return {'mgs': 'The token has expired'}, 422


@api.errorhandler(jwt_extended_exception.InvalidHeaderError)
def handle_invalid_header_error(e):
    return {'mgs': str(e)}, 422


@api.errorhandler(jwt_exception.InvalidTokenError)
def handle_invalid_token_error(e):
    return {'mgs': str(e)}, 422


@api.errorhandler(jwt_extended_exception.JWTDecodeError)
def handle_jwt_decode_error(e):
    return {'mgs': str(e)}, 422


@api.errorhandler(jwt_extended_exception.WrongTokenError)
def handle_wrong_token_error(e):
    return {'mgs': str(e)}, 422


@api.errorhandler(jwt_extended_exception.RevokedTokenError)
def handle_revoked_token_error(e):
    return {'mgs': 'Token has been revoked'}, 401


@api.errorhandler(jwt_extended_exception.FreshTokenRequired)
def handle_fresh_token_required(e):
    return {'mgs': 'Fresh token required'}, 401


@api.errorhandler(jwt_extended_exception.UserLookupError)
def handler_user_load_error(e):
    # The identity is already saved before this exception was raised,
    # otherwise a different exception would be raised, which is why we
    # can safely call get_jwt_identity() here
    identity = get_jwt_identity()
    return {'mgs': "Error loading the user {}".format(identity)}, 401


@api.errorhandler(jwt_extended_exception.UserClaimsVerificationError)
def handle_failed_user_claims_verification(e):
    return {'mgs': 'User claims verification failed'}, 400



from setuptools import setup, find_packages
SERVICE_NAME = 'microservice-template-core'
SERVICE_NAME_NORMALIZED = SERVICE_NAME.replace('-', '_')

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

tests_require = [],

setup(
    name=SERVICE_NAME,
    version='2.2.10',
    description="Microservice template core",
    url='',
    include_package_data=True,
    author='',
    author_email='',
    classifiers=[
    ],
    keywords=[],
    packages=find_packages(),
    install_requires=requirements,
    tests_require=tests_require,
    entry_points={
        'console_scripts': [
            f'{SERVICE_NAME} = {SERVICE_NAME_NORMALIZED}.core:main',
        ]
    },
    zip_safe=False,
    cmdclass={},
    data_files=[]
)



# from opentelemetry import trace
# from opentelemetry.exporter.jaeger.thrift import JaegerExporter
# from opentelemetry.sdk.trace import TracerProvider
# from opentelemetry.sdk.trace.export import BatchSpanProcessor
# from microservice_template_core.settings import TracerConfig, ServiceConfig
# tracer = None
#
#
# def get_tracer():
#     global tracer
#     if not tracer:
#         trace.set_tracer_provider(TracerProvider())
#         jaeger_exporter = JaegerExporter(
#             agent_host_name=TracerConfig.JAEGER_SERVER,
#             agent_port=TracerConfig.JAEGER_PORT,
#         )
#
#         trace.get_tracer_provider().add_span_processor(
#             BatchSpanProcessor(jaeger_exporter)
#         )
#
#         tracer = trace.get_tracer(__name__)
#     return tracer
